@@ -392,8 +392,9 @@ acceptport (int *sock)
   socklen_t addrlen = sockaddr_len ();
 
 #ifdef HAVE_SELECT
-  if (select_fd (msock, opt.connect_timeout, 0) <= 0)
-    return ACCEPTERR;
+  if (opt.connect_timeout)
+    if (select_fd (msock, opt.connect_timeout, 0) <= 0)
+      return ACCEPTERR;
 #endif
   if ((*sock = accept (msock, addr, &addrlen)) < 0)
     return ACCEPTERR;
